# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import qtawesome as qta
from qtpy import QtWidgets
from prettyqt import core, gui


class RadioButton(QtWidgets.QRadioButton):

    value_changed = core.Signal(bool)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.toggled.connect(self.value_changed)

    def __repr__(self):
        return f"RadioButton: {self.__getstate__()}"

    def __getstate__(self):
        return dict(object_name=self.objectName(),
                    checkable=self.isCheckable(),
                    icon=gui.Icon(self.icon()),
                    checked=self.isChecked(),
                    text=self.text(),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip(),
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        super().__init__()
        self.setObjectName(state.get("object_name", ""))
        self.set_icon(state["icon"])
        self.setChecked(state.get("checked", False))
        self.setText(state.get("text", ""))
        self.setEnabled(state.get("enabled", True))
        self.setCheckable(state["checkable"])
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def __bool__(self):
        return self.isChecked()

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_icon(self, icon):
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.setIcon(icon)

    def get_value(self) -> bool:
        return self.isChecked()

    def set_value(self, value: bool):
        self.setChecked(value)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = RadioButton("This is a test")
    widget.set_icon("mdi.timer")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import datetime

from qtpy import QtWidgets

from prettyqt import core


class DateEdit(QtWidgets.QDateEdit):

    value_changed = core.Signal(datetime.date)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setCalendarPopup(True)
        self.dateChanged.connect(self.date_changed)

    def date_changed(self, date):
        self.value_changed.emit(self.get_date())

    def __getstate__(self):
        return dict(calendar_popup=self.calendarPopup(),
                    date=self.get_date(),
                    display_format=self.displayFormat(),
                    range=(self.min_date(), self.max_date()),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip(),
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__(state["date"])
        self.setEnabled(state.get("enabled", True))
        self.setDisplayFormat(state["display_format"])
        self.set_range(*state["range"])
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def set_value(self, value):
        return self.setDate(value)

    def set_range(self, lower: datetime.date, upper: datetime.date):
        self.setToolTip(f"{lower} <= x <= {upper}")
        self.setDateRange(lower, upper)

    def get_value(self) -> datetime.date:
        return self.get_date()

    def get_date(self) -> datetime.date:
        try:
            return self.date().toPython()
        except (TypeError, AttributeError):
            return self.date().toPyDate()

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def min_date(self) -> datetime.date:
        try:
            return self.minimumDate().toPython()
        except (TypeError, AttributeError):
            return self.minimumDate().toPyDate()

    def max_date(self) -> datetime.date:
        try:
            return self.maximumDate().toPython()
        except (TypeError, AttributeError):
            return self.maximumDate().toPyDate()


if __name__ == "__main__":
    from prettyqt import widgets
    app = widgets.app()
    widget = DateEdit()
    widget.show()
    app.exec_()
